import numpy as np
import matplotlib.pyplot as plt
from scipy.stats import beta

# Define the true probabilities of success for each drug (hidden in the simulation)
true_p_A = 0.8
true_p_B = 0.5


def simulate_drug_A():
    """Simulate the effectiveness of a drug with a hidden true probability of success."""
    return np.random.rand() < true_p_A

def simulate_drug_B():
    """Simulate the effectiveness of a drug with a hidden true probability of success."""
    return np.random.rand() < true_p_B






# Number of trials
num_trials = 100

# Prior parameters for Beta distributions (assuming uninformative priors)
alpha_A, beta_A = 1, 1
alpha_B, beta_B = 1, 1


# Simulate the trials
for _ in range(num_trials):
    # Sample from the posterior distributions
    sample_A = np.random.beta(alpha_A, beta_A)
    sample_B = np.random.beta(alpha_B, beta_B)

    # Lists to keep track of successes and failures
    successes_A, failures_A = 0, 0
    successes_B, failures_B = 0, 0

    # Choose the drug with the higher sample
    if sample_A > sample_B:
        reward = simulate_drug_A()
        if reward:
            successes_A += 1
        else:
            failures_A += 1
    else:
        reward = simulate_drug_B()
        if reward:
            successes_B += 1
        else:
            failures_B += 1
    
    # Update the priors
    alpha_A += successes_A
    beta_A += failures_A
    alpha_B += successes_B
    beta_B += failures_B
    
# Plotting the results
x = np.linspace(0, 1, 100)
posterior_A = beta(alpha_A, beta_A).pdf(x)
posterior_B = beta(alpha_B, beta_B).pdf(x)

plt.plot(x, posterior_A, label="Drug A Posterior")
plt.plot(x, posterior_B, label="Drug B Posterior")
plt.title("Posterior Distributions of Drug A and Drug B")
plt.legend()
plt.show()

print(f"Total successes for Drug A: {alpha_A - 1}")
print(f"Total successes for Drug B: {alpha_B - 1}")

print(f"We tried drug A {alpha_A + beta_A - 2}")
print(f"We tried drug B {alpha_B + beta_B - 2}")